import { Inject, Injectable, InjectionToken, NgModule, NgZone, Optional } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * The token used to allow injection of the {\@link StorageOptions} interface. For more information visit the
 * {\@link https://angular.io/docs/ts/latest/guide/dependency-injection.html#interface angular2 docs}.
 */
let STORAGE_OPTIONS = new InjectionToken('StorageOptions');
/**
 * An object that SERializes and DESeriallizes values for storage. The default implementation for the SerDes object is:
 * `{ provide: SERDES_OBJECT, useValue: { stringify: JSON.stringify, parse: JSON.parse } }` which is essentially the
 * `{\@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/JSON window.JSON}` object.
 * Due to the fact that the JSON object is also an interface in TypeScript, it couldn't be used as-is. This is the reason
 * the default implementation defines the `stringify` and `parse` methods individually instead of just using the JSON object
 * as the value.
 *
 * Should you want to implement customized serialization and deserialization for something like encrypted storage, simply
 * provide new methods for stringify and parse and you're good to go.
 */
let SERDES_OBJECT = new InjectionToken('SerdesObject');
/**
 * Defines options used by the storage classes to determine how they will interact. This will be injected into any Storage classes
 * created with or after creation of the options. Here is a comon usage of StorageOptions:
 *
 *     import {Component} from "\@angular/core";
 *     import {ConfigureStorage} from "h5webstorage";
 *
 *     \@Component({
 * 	     providers:[ConfigureStorage({ prefix: ''})]
 *     })
 *     export class AppModule{}
 *
 *
 * {\@link ConfigureStorage} creates a provider for your configuration options which is injected into the next Storage class instantiated.
 * Since these options were added to the 'AppModule', it will apply globally and all {\@link LocalStorage} and {\@link SessionStorage} will
 * have these options injected unless overridden by a new set of options.
 *
 * Remember, {\@link https://angular.io angular2} has an injection heirarchy and multiple providers for one type can be created. Lets explore
 * how this might work. Using our example above, the prefix '/' is used on all keys for the entire application. This will prevent
 * h5websetorage from assuming it controls all the keys in storage. Let's also assume we have created a wrapper around the `LocalStorage`
 * object to apply business logic to the data and a subset of the keys needs to be read-only admin stuff. How can we set the values if the
 * wrapper of those values are read-only? Here is how I would solve this problem:
 *
 *     import {Component} from "\@angular/core";
 *     import {LocalStorage, ConfigureStorage} from "h5webstorage";
 *     import {AppSettings} from "./myApp.settings";
 *     \@Component({
 *     	providers: [LocalStorage, ConfigureStorage({ prefix: '/admin/'})]
 *     })
 *     export class AdminPanel{
 *     	constructor(localStorage: LocalStorage, settings: AppSettings){ ... }
 *     }
 *
 * Let me explain what just happened, AdminPanel is a component under AppModule. AppSettings is the global application setting that handles
 * business logic for the storage which has the read-only admin area. AppSettings can see all the application keys that begin with '/' so
 * that includes '/admin/'. AdminPanel takes advantage of the injection heirarcy in angular2 by asking for a new instance of `LocalStorage`.
 * If we stopped there, it would work the exact same way as the `LocalStorage` in AppSettings which is why there is a new `ConfigureStorage`
 * call in the providers array also. This will tell angular2 to inject these new options into the new `LocalStorage` instance which will be
 * scoped to only the '/admin/' keys. `LocalStorage` has no restrictions on which keys it can modify within its scope. Creating, updating,
 * modifying or deleting a key in `LocalStorage` within AdminPanel will reflect immediately in the AppSettings instance.
 * @record
 */

/** @enum {number} */
const KeyDirection = {
    From: 0,
    To: 1,
};
KeyDirection[KeyDirection.From] = "From";
KeyDirection[KeyDirection.To] = "To";
/**
 * BaseStorage is where all the implementation of the storage system resides. This can be used to make custom storage objects if needed
 * but more than likely, `LocalStorage` and `SessionStorage` will handle everything you need and creating services around those will
 * be easier than creating a new storage object. But for those situations where something else is needed, this is the class for you.
 *
 * NOTE: BaseStorage is an abstract class but the documentation generator makes no distintion.
 * @abstract
 */
class BaseStorageService {
    /**
     * Initializes the storage system by attaching to the angular2 change detection system and the Storage event system. Then
     * syncronizes the object with data from storage while respecting the options set. {\@link LocalStorage}
     *
     * @param {?} ngZone - The Zone handling angular 2 change detection.
     * @param {?} storage - An object that implements the Storage interface. Generally [localStorage][] or [sessionStorage][]
     * [localStorage]: https://developer.mozilla.org/en-US/docs/Web/API/Window/localStorage
     * [sessionStorage]: https://developer.mozilla.org/en-US/docs/Web/API/Window/sessionStorage
     * @param {?} transformer - An object that implements the JSON interface. This is injected using the {\@link SERDES_OBJECT}
     * @param {?=} options - Contains the options that will be used with the instance
     */
    constructor(ngZone, storage, transformer, options) {
        // options = Object.assign({ prefix: "", transformer: JSON }, options);
        this.setProperty('options', options);
        this.setProperty('transformer', transformer);
        this.setProperty('storage', storage);
        this.UpdateFromStorage();
        const /** @type {?} */ subscription = ngZone.onMicrotaskEmpty.subscribe(() => {
            const /** @type {?} */ fromStorage = this.getProperty('fromStorage');
            fromStorage ? this.UpdateFromStorage() : this.WriteToStorage();
        });
        const /** @type {?} */ listener = (event) => event.storageArea === storage && this.setProperty('fromStorage', true);
        window.addEventListener('storage', listener);
        this.setProperty('subscription', subscription);
        this.setProperty('listener', listener);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.getProperty('subscription').unsubscribe();
        const /** @type {?} */ listener = this.getProperty('listener');
        window.removeEventListener('storage', listener);
    }
    /**
     * Retrieves data stored is storage.
     * @param {?} key - The identifier used to locate data in storage.
     * @return {?}
     */
    getItem(key) {
        return this[key];
    }
    /**
     * Stores data in storage
     * @param {?} key - The identifier to associate stored data with
     * @param {?} value - The data to place in storage
     * @return {?}
     */
    setItem(key, value) {
        try {
            // since the value of set item has to be a string, the value may already be stringified Json.
            // so we parse it to allow the WriteToStorage function to properly stringify object values.
            this[key] = this.deserialize(value);
        }
        catch (/** @type {?} */ e) {
            this[key] = value;
        }
    }
    /**
     * Deletes data stored in storage
     * @param {?} key - The identifier used to locate the data to be deleted
     * @return {?}
     */
    removeItem(key) {
        delete this[key];
    }
    /**
     * Removes all keys managed by the storage object.
     * @return {?}
     */
    clear() {
        Object.keys(this).forEach((key) => {
            delete this[key];
        });
    }
    /**
     * Retrieves an identifier
     * @param {?} index - The position at which to look for an identifier. Index be at least zero
     * and must be less than length.
     * @return {?}
     */
    key(index) {
        return Object.keys(this)[index];
    }
    /**
     * The number of values this instance is managing
     * @return {?}
     */
    get length() {
        return Object.keys(this).length;
    }
    /**
     * Retrieves data from the object stored as a property.
     * @template T
     * @param {?} name
     * @return {?}
     */
    getProperty(name) {
        return name in this ? this[name] : undefined;
    }
    /**
     * Stores data in a private private property that will not be stored in storage.
     * This is necessary because creating a class property results in an extra key in
     * storage.
     * @param {?} name
     * @param {?} value
     * @return {?}
     */
    setProperty(name, value) {
        if (!(name in this)) {
            Object.defineProperty(this, name, { writable: true });
        }
        this[name] = value;
    }
    /**
     * @param {?} name
     * @param {?} dir
     * @return {?}
     */
    normalizeStorageKey(name, dir) {
        const /** @type {?} */ options = this.getProperty('options');
        return dir === KeyDirection.From ? name.replace(options.prefix, '') : options.prefix + name;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    serialize(value) {
        const /** @type {?} */ transformer = this.getProperty('transformer');
        return transformer.stringify(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    deserialize(value) {
        const /** @type {?} */ transformer = this.getProperty('transformer');
        return transformer.parse(value);
    }
    /**
     * @return {?}
     */
    WriteToStorage() {
        const /** @type {?} */ prevValue = this.getProperty('prevValue');
        const /** @type {?} */ currValue = this.serialize(this);
        // tslint:disable-next-line:triple-equals
        if (prevValue != currValue) {
            const /** @type {?} */ storage = this.getProperty('storage');
            const /** @type {?} */ prevStorage = this.deserialize(prevValue);
            Object.keys(this).forEach((key) => {
                const /** @type {?} */ _key = this.normalizeStorageKey(key, KeyDirection.To);
                const /** @type {?} */ value = this[key];
                if (typeof this[key] !== 'undefined') {
                    storage.setItem(_key, this.serialize(this[key]));
                    delete prevStorage[key];
                }
            }, this);
            for (const /** @type {?} */ key in prevStorage) {
                if (prevStorage.hasOwnProperty(key)) {
                    storage.removeItem(this.normalizeStorageKey(key, KeyDirection.To));
                }
            }
            this.setProperty('prevValue', this.serialize(this));
        }
    }
    /**
     * @return {?}
     */
    UpdateFromStorage() {
        const /** @type {?} */ options = this.getProperty('options');
        const /** @type {?} */ storage = this.getProperty('storage');
        const /** @type {?} */ tmp = Object.assign({}, this);
        Object.keys(storage).forEach((key) => {
            if (!key.startsWith(options.prefix)) {
                return;
            }
            const /** @type {?} */ _key = this.normalizeStorageKey(key, KeyDirection.From);
            try {
                delete tmp[_key];
                this[_key] = this.deserialize(storage[key]);
            }
            catch (/** @type {?} */ e) {
                this[_key] = options.serializeOnException ?
                    this.serialize(storage[key]) :
                    null;
            }
        });
        for (const /** @type {?} */ key in tmp) {
            if (tmp.hasOwnProperty(key)) {
                delete this[this.normalizeStorageKey(key, KeyDirection.From)];
            }
        }
        this.setProperty('prevValue', this.serialize(this));
        this.setProperty('fromStorage', false);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Token used to inject an object as the storage backend of the SessionStorage object. By default, the storage
 * backend is the native localStorage object but can be substituted to allow for testing or customized storage
 * like to remote storage.
 */
let LOCAL_STORAGE_OBJECT = new InjectionToken('localstorage');
/**
 * Represents the native {\@link https://developer.mozilla.org/en-US/docs/Web/API/Window/localStorage localStorage} object.
 * Can be injected into a component that needs access to localStorage.
 */
class LocalStorageService extends BaseStorageService {
    /**
     * @param {?} ngZone
     * @param {?} storage
     * @param {?} transformer
     * @param {?=} options
     */
    constructor(ngZone, storage, transformer, options) {
        super(ngZone, storage, transformer, options);
    }
}
LocalStorageService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
LocalStorageService.ctorParameters = () => [
    { type: NgZone, },
    { type: undefined, decorators: [{ type: Inject, args: [LOCAL_STORAGE_OBJECT,] },] },
    { type: JSON, decorators: [{ type: Inject, args: [SERDES_OBJECT,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [STORAGE_OPTIONS,] }, { type: Optional },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Token used to inject an object as the storage backend of the SessionStorage object. By default, the storage
 * backend is the native sessionStorage object but can be substituted to allow for testing or customized storage
 * like to remote storage.
 */
let SESSION_STORAGE_OBJECT = new InjectionToken('sessionstorage');
/**
 * Represent the native {\@link https://developer.mozilla.org/en-US/docs/Web/API/Window/sessionStorage sessionStorage} object.
 * Can be injected into a component that needs access to sessionStorage.
 */
class SessionStorageService extends BaseStorageService {
    /**
     * @param {?} ngZone
     * @param {?} storage
     * @param {?} transformer
     * @param {?=} options
     */
    constructor(ngZone, storage, transformer, options) {
        super(ngZone, storage, transformer, options);
    }
}
SessionStorageService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
SessionStorageService.ctorParameters = () => [
    { type: NgZone, },
    { type: undefined, decorators: [{ type: Inject, args: [SESSION_STORAGE_OBJECT,] },] },
    { type: JSON, decorators: [{ type: Inject, args: [SERDES_OBJECT,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [STORAGE_OPTIONS,] }, { type: Optional },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Makes the  LocalStorage and SessionStorage objects available throught the application. Should be
 * added to the RootModule imports list.
 */
class WebStorageModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: WebStorageModule,
            providers: [
                { provide: LOCAL_STORAGE_OBJECT, useValue: localStorage },
                { provide: SESSION_STORAGE_OBJECT, useValue: sessionStorage },
                { provide: SERDES_OBJECT, useValue: { stringify: JSON.stringify, parse: JSON.parse } },
                ConfigureStorage({ prefix: '' })
            ]
        };
    }
}
WebStorageModule.decorators = [
    { type: NgModule, args: [{
                providers: [LocalStorageService, SessionStorageService]
            },] },
];
/** @nocollapse */
WebStorageModule.ctorParameters = () => [];
/**
 * Creates a provider for the StorageOptions
 * @param {?} options - A {\@link StorageOptions} object
 * @return {?}
 */
function ConfigureStorage(options) {
    return { provide: STORAGE_OPTIONS, useValue: options };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {...?} params
 * @return {?}
 */
function StorageProperty(...params) {
    return (target, decoratedPropertyName) => {
        let /** @type {?} */ options;
        if (params[0] && typeof params[0] === 'object') {
            options = Object.assign({}, { storageKey: decoratedPropertyName, storage: 'Local', readOnly: false }, params[0]);
        }
        else {
            options = Object.assign({}, { storageKey: params[0] || decoratedPropertyName, storage: params[1] || 'Local', readOnly: false });
        }
        let /** @type {?} */ storeObject = null;
        const /** @type {?} */ initializing = !options.readOnly;
        /*in the current context, 'this' is the module containing the StorageProperty but what we
                 * actually want is the object with the proerty we're defining. So we create this function
                 * to be bound with the correct 'this' so we can have access to its properies. Of which,
                 * one should be an instance of LocalStorage or SessionStorage
                 */
        const /** @type {?} */ findStore = function (obj) {
            if (!storeObject) {
                const /** @type {?} */ storeKey = Object.keys(obj).find(function (key) {
                    return options.storage === 'Local' ? obj[key] instanceof LocalStorageService : obj[key] instanceof SessionStorageService;
                });
                // tslint:disable-next-line:no-unused-expression
                storeKey && (storeObject = obj[storeKey]);
                if (!storeObject) {
                    throw new Error('Object must have a property that is an instance of ' + options.storage + 'Storage.');
                }
            }
            return storeObject;
        };
        const /** @type {?} */ initialize = function (obj) {
            // by adding a non-enumerable field to the object, we know if it's new or not and won't disrupt the developers code
            Object.defineProperty(obj, '_' + options.storageKey, {
                enumerable: false,
                value: undefined,
                writable: true
            });
        };
        const /** @type {?} */ propertyObj = {
            get: function () {
                initialize(this);
                findStore(this);
                return storeObject.getItem(options.storageKey);
            },
            enumerable: false,
        };
        // if requesting a readOnly property don't create the set
        if (!options.readOnly) {
            propertyObj['set'] = function (value) {
                if (!(/** @type {?} */ (this)).hasOwnProperty('_' + options.storageKey)) {
                    initialize(this);
                    findStore(this);
                    const /** @type {?} */ storedValue = storeObject.getItem(options.storageKey);
                    if (typeof storedValue !== 'undefined') {
                        return;
                    }
                }
                storeObject.setItem(options.storageKey, value);
            };
        }
        Object.defineProperty(target, decoratedPropertyName, propertyObj);
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { LocalStorageService as LocalStorage, LOCAL_STORAGE_OBJECT, SessionStorageService as SessionStorage, SESSION_STORAGE_OBJECT, WebStorageModule, ConfigureStorage, STORAGE_OPTIONS, SERDES_OBJECT, BaseStorageService, StorageProperty };
//# sourceMappingURL=h5webstorage.js.map
